@@ -1,6 +1,5 @@
 /* $Copyright: $
  * Copyright (c) 1996 - 2022 by Steve Baker (ice@mama.indstate.edu)
- * All Rights reserved
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +17,8 @@
  */
 #include "tree.h"
 
-extern bool dflag, Fflag, aflag, fflag, pruneflag;
-extern bool noindent, force_color, flimit, matchdirs;
+extern bool dflag, Fflag, aflag, fflag, pruneflag, gitignore, showinfo;
+extern bool noindent, force_color, matchdirs, fflinks;
 extern bool reverse;
 extern int pattern, ipattern;
 
@@ -32,6 +31,9 @@ extern char *endcode;
 
 extern char *file_comment, *file_pathsep;
 
+/* 64K paths maximum */
+#define MAXPATH	64*1024
+
 enum ftok { T_PATHSEP, T_DIR, T_FILE, T_EOP };
 
 char *nextpc(char **p, int *tok)
@@ -39,7 +41,7 @@ char *nextpc(char **p, int *tok)
   static char prev = 0;
   char *s = *p;
   if (!**p) {
-    *tok = T_EOP;	// Shouldn't happen.
+    *tok = T_EOP;	/* Shouldn't happen. */
     return NULL;
   }
   if (prev) {
@@ -71,7 +73,7 @@ struct _info *newent(char *name) {
   return n;
 }
 
-// Should replace this with a Red-Black tree implementation or the like
+/* Should replace this with a Red-Black tree implementation or the like */
 struct _info *search(struct _info **dir, char *name)
 {
   struct _info *ptr, *prev, *n;
@@ -108,12 +110,23 @@ void freefiletree(struct _info *ent)
  * Recursively prune (unset show flag) files/directories of matches/ignored
  * patterns:
  */
-struct _info **fprune(struct _info *head, bool matched, bool root)
+struct _info **fprune(struct _info *head, char *path, bool matched, bool root)
 {
   struct _info **dir, *new = NULL, *end = NULL, *ent, *t;
-  int show, count = 0;
+  struct comment *com;
+  struct ignorefile *ig = NULL;
+  struct infofile *inf = NULL;
+  char *cur, *fpath = xmalloc(sizeof(char) * MAXPATH);
+  int i, show, count = 0;
+
+  strcpy(fpath, path);
+  cur = fpath + strlen(fpath);
+  *(cur++) = '/';
+
+  push_files(path, &ig, &inf);
 
   for(ent = head; ent != NULL;) {
+    strcpy(cur, ent->name);
     if (ent->tchild) ent->isdir = 1;
 
     show = 1;
@@ -129,7 +142,15 @@ struct _info **fprune(struct _info *head, bool matched, bool root)
       }
     }
     if (pruneflag && !matched && ent->isdir && ent->tchild == NULL) show = 0;
-    if (show && ent->tchild != NULL) ent->child = fprune(ent->tchild, matched, FALSE);
+    if (gitignore && filtercheck(path, ent->name, ent->isdir)) show = 0;
+    if (show && showinfo && (com = infocheck(path, ent->name, inf != NULL, ent->isdir))) {
+      for(i = 0; com->desc[i] != NULL; i++);
+      ent->comment = xmalloc(sizeof(char *) * (i+1));
+      for(i = 0; com->desc[i] != NULL; i++) ent->comment[i] = scopy(com->desc[i]);
+      ent->comment[i] = NULL;
+    }
+    if (show && ent->tchild != NULL) ent->child = fprune(ent->tchild, fpath, matched, FALSE);
+
 
     t = ent;
     ent = ent->next;
@@ -152,13 +173,17 @@ struct _info **fprune(struct _info *head, bool matched, bool root)
 
   if (topsort) qsort(dir,count,sizeof(struct _info *),topsort);
 
+  if (ig != NULL) ig = pop_filterstack();
+  if (inf != NULL) inf = pop_infostack();
+  free(fpath);
+
   return dir;
 }
 
 struct _info **file_getfulltree(char *d, u_long lev, dev_t dev, off_t *size, char **err)
 {
   FILE *fp = (strcmp(d,".")? fopen(d,"r") : stdin);
-  char *path, *spath, *s;
+  char *path, *spath, *s, *link;
   long pathsize;
   struct _info *root = NULL, **cwd, *ent;
   int l, tok;
@@ -168,17 +193,23 @@ struct _info **file_getfulltree(char *d, u_long lev, dev_t dev, off_t *size, cha
     fprintf(stderr,"Error opening %s for reading.\n", d);
     return NULL;
   }
-  // 64K paths maximum
-  path = xmalloc(sizeof(char *) * (pathsize = (64 * 1024)));
+  path = xmalloc(sizeof(char *) * (pathsize = MAXPATH));
 
   while(fgets(path, pathsize, fp) != NULL) {
     if (file_comment != NULL && strcmp(path,file_comment) == 0) continue;
     l = strlen(path);
-    while(l && isspace(path[l-1])) path[--l] = '\0';
+    while(l && (path[l-1] == '\n' || path[l-1] == '\r')) path[--l] = '\0';
     if (l == 0) continue;
 
     spath = path;
     cwd = &root;
+
+    link = fflinks? strstr(path, " -> ") : NULL;
+    if (link) {
+      *link = '\0';
+      link += 4;
+    }
+    ent = NULL;
     do {
       s = nextpc(&spath, &tok);
       if (tok == T_PATHSEP) continue;
@@ -186,22 +217,31 @@ struct _info **file_getfulltree(char *d, u_long lev, dev_t dev, off_t *size, cha
 	case T_PATHSEP: continue;
 	case T_FILE:
 	case T_DIR:
-	  // Should probably handle '.' and '..' entries here
+	  /* Should probably handle '.' and '..' entries here */
 	  ent = search(cwd, s);
-	  // Might be empty, but should definitely be considered a directory:
+	  /* Might be empty, but should definitely be considered a directory: */
 	  if (tok == T_DIR) {
 	    ent->isdir = 1;
 	    ent->mode = S_IFDIR;
 	  } else {
 	    ent->mode = S_IFREG;
 	  }
+
 	  cwd = &(ent->tchild);
 	  break;
       }
     } while (tok != T_FILE && tok != T_EOP);
+
+    if (link) {
+      ent->isdir = 0;
+      ent->mode = S_IFLNK;
+      ent->lnk = strdup(link);
+    }
   }
   if (fp != stdin) fclose(fp);
 
-  // Prune accumulated directory tree:
-  return fprune(root, FALSE, TRUE);
+  free(path);
+
+  /* Prune accumulated directory tree: */
+  return fprune(root, "", FALSE, TRUE);
 }
